unit Auto;
(*
  Esimerkki
   1) TImageListin kytst
   2) TPaintBoxin perimisest
   3) TBitMapin kytst (mm. peilaaminen y-akselin suhteen)
   4) Viestin lhettmisest (Event) omistajalle
       - auto lhett aina liikahtaessaan viestin
       - auto lhettn viestin kun se kntyy

   Vesa Lappalainen 23.10.1997

   Changed  : 5.11.1999/vl
    + peritty TSpritest
   Changed  : 14.11.1999/vl
    + korjattu React-metodia
    + korjattu PaintTo-metodia siten, etta suunta ptetn suunnasta, ei dx:st
    + listty AutoReact perittvksi paremmin
*)
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls,sprite;

type
  TAuto = class;

  TOnAutoMove  = procedure (sender:TAuto) of object;
  TOnChangeDir = procedure (sender:TObject;dir:integer) of object;

  TAuto = class(TSprite)
  private
    FVauhti : integer;
    FSuunta : integer;
    dx,dy   : integer;
    FOnAutoMove : TOnAutoMove;
    FOnChangeDir : TOnChangeDir;
  protected
    FBitName : String;
    FImageList : TImageList;
  public
    constructor Create(AOwner:TComponent); override;
    destructor  Destroy; override;
    procedure SetBitName(s:string); virtual;
    procedure SetSuunta(i:integer); virtual;
    procedure SetVauhti(i:integer); virtual;
    function DoJob:boolean; override;
    function PaintTo(canvas:TCanvas):boolean; override;
    function React(sprite:TSprite):boolean; override;
    function AutoReact(sprite:TSprite):boolean; virtual;
  published
    property BitName : string read FBitName write SetBitName;
    property Vauhti : integer read FVauhti write SetVauhti default 0;
    property Suunta : integer read FSuunta write SetSuunta default 1;
    property OnAutoMove : TOnAutoMove read FOnAutoMove write FOnAutoMove;
    property OnChangeDir : TOnChangeDir read FOnChangeDir write FOnChangeDir;
  end;

procedure Register;

implementation

constructor TAuto.Create(AOwner:TComponent);
begin
  inherited;
  FImageList := TImageList.Create(self);
  FVauhti := 0;
  FSuunta := 1;
  dy := 0;
  dx := FVauhti;
  FOnAutoMove := NIL;
  FOnChangeDir := NIL;
end;

destructor TAuto.Destroy;
begin
  FImageList.Free;
  inherited;
end;


procedure TAuto.SetSuunta(i:integer);
begin
  if ( i <> FSuunta ) then if Assigned(FOnChangeDir) then FOnChangeDir(self,i);
  FSuunta := i;
  dx := Suunta * Vauhti;
//  Paint;                                         // Varmistetaan ett piirretn
end;

procedure TAuto.SetVauhti(i:integer);
begin
  FVauhti := i;
  dx := Suunta * Vauhti;
end;

procedure TAuto.SetBitName(s:string);
// Kun bittikartan tiedoston nimi vaihdetaan, pyyhitn vanhat kuvat pois ja
// luodaan uudet kuvat.  Vasemmalle menev kuva saadaan peilaamalla oikealle
// menevt kuva y-akselin suhteen.
var rect,rect2 : TRect;
    lbmp,rbmp :  TBitmap; // Vasemmalle ja oikealle menevt kuvat
begin
  FImageList.Clear;                              // Pyyhitn vanhat kuvat pois
  if ( s = '' ) then begin {Paint;} exit; end;     // Jos ei nime, ei kuvia
  rbmp := TBitmap.Create;                        // Luodaan tilap. bittikartat
  lbmp := TBitmap.Create;                        // Oik. ja avsemmalle

  FBitName := s;                                 // Muistetaan kuvan nimi
  rbmp.LoadFromFile(s);                          // Luetaan Oik. kuva tied.

  lbmp.Assign(rbmp);                             // Tehdn vas. identtinen
  rect.Left   := 0;                              // Apusuorakaiteet kop.
  rect.Top    := 0;
  rect.Right  := rbmp.Width;
  rect.Bottom := rbmp.Height;
  rect2       := rect;                           // rect2 on rectin peilikuva
  rect2.Right := 0;
  rect2.Left  := rect.Right;
  lbmp.Canvas.CopyRect(rect2,rbmp.Canvas, rect); // lbmp on rbmp:n peilikuva
//  FRightBitmap.Canvas.StretchDraw(rect2,FLeftBitmap); // Toimisi mys

  w  := rbmp.Width;                          // Oma koko bitmapin kokoiseksi
  h := rbmp.Height;
  FImageList.Width  := rbmp.Width;                    // Samoin kuvalistan
  FImageList.Height := rbmp.Height;
  FImageList.AddMasked(rbmp,rbmp.TransparentColor); // Kuvalistan paik. 0 rbmp
  FImageList.AddMasked(lbmp,lbmp.TransparentColor); //                  1 lbmp
 // Paint;
  lbmp.Free;                                     // Poistetaan tilap. kuvat
  rbmp.Free;
end;

function TAuto.DoJob : boolean;
begin
  inherited DoJob;
  if ( y + dy < 0 ) then dy := abs(dy);
  if ( y + dy >= Parent.ClientHeight-h ) then dy := -abs(dy);
  if ( x + dx < 0 ) then suunta := 1;
  if ( x + dx >= Parent.ClientWidth-w ) then suunta := -1;
  y  := y + dy;
  x := x + dx;
  if Assigned(FOnAutoMove) then FOnAutoMove(self);
  Result := true;
end;

function TAuto.PaintTo(canvas: TCanvas): boolean;
var index : integer;
begin
  index := 0; if ( suunta < 0 ) then index := 1;  // Kumpiko kuva pit piirt
  FImageList.Draw(Canvas,ix,iy,index);
  inherited PaintTo(canvas);
  Result := true;
end;


procedure Register;
begin
  RegisterComponents('GKO99', [TAuto]);
end;


function TAuto.AutoReact(sprite: TSprite): boolean;
begin
  if ( sprite is TAuto ) then begin
    parent.UnJoin(self,true);
    parent.UnJoin(sprite,true);
  end;
  Result := true;
end;

function TAuto.React(sprite: TSprite): boolean;
begin
  Result := inherited React(sprite);
  if ( Result ) then exit;
  Result := AutoReact(sprite);
end;


end.
