{------------------------------------------------------------------------------}
{
   Unit Name: traffic
   Purpose  : Implent traffic ligths 
   Author   : Vesa Lappalainen
   Date     : 14.11.1999
   Changed  :

   ToDo     :
}
{------------------------------------------------------------------------------}

unit traffic;


interface
uses classes,graphics,sprite;

type
  TLightState = (ls_Red,ls_Yellow,ls_Green);

  TTrafficLights = class(TSprite)
  private
    FState: TLightState;
    FCounter : integer;
    FInterval: integer;
    FRedProcent: integer;
    FGreenProcent: integer;
    procedure SetInterval(const Value: integer);
    procedure SetRedProcent(const Value: integer);
    procedure SetGreenProcent(const Value: integer);
  public
    constructor Create(AOwner:TComponent); override;
    function DoJob:boolean; override;
    function PaintTo(canvas:TCanvas):boolean; override;
  published
    property State : TLightState read FState;
    property Interval : integer read FInterval write SetInterval;
    property RedProcent : integer read FRedProcent write SetRedProcent;
    property GreenProcent : integer read FGreenProcent write SetGreenProcent;
  end;


implementation

{ TTrafficLights }

const lights : array[false..true,ls_Red..ls_Green] of TColor = (
  (clBlack,clBlack,clBlack),
  (clRed,clYellow,clLime)
  );

constructor TTrafficLights.Create(AOwner: TComponent);
begin
  inherited;
  FInterval := 10;
  FRedProcent := 40;
  FGreenProcent := 40;
  FCounter := 0;
  w := 30;
  h := 40;
end;

function TTrafficLights.DoJob: boolean;
var old_state : TLightState;
begin
  old_state := State;
  inc(FCounter);
  if ( FCounter >= Interval * RedProcent div 100) then FState := ls_Yellow;
  if ( FCounter >= Interval - (Interval * GreenProcent div 100) ) then FState := ls_Green;
  if ( FCounter >= Interval ) then begin FState := ls_Red; FCounter := 0; end;
  Result := old_state <> State;
end;

function TTrafficLights.PaintTo(canvas: TCanvas): boolean;
var lx,ly,r : integer; s : TLightState;
begin
  canvas.Brush.Color := clBlack;
  canvas.Rectangle(ix+iw div 2-2,iy,ix+iw div 2+2,iy+ih);
  r := ih div 10;
  lx := ix + iw div 2 - r;
  ly := iy;
  for s := ls_Red to ls_Green do begin
    canvas.Brush.Color := lights[State=s,s];
    canvas.Ellipse(lx,ly,lx+2*r,ly+2*r);
    ly := ly + 2*r;
  end;
  Result := true;
end;

procedure TTrafficLights.SetGreenProcent(const Value: integer);
begin
  FGreenProcent := Value;
end;

procedure TTrafficLights.SetInterval(const Value: integer);
begin
  FInterval := Value;
end;

procedure TTrafficLights.SetRedProcent(const Value: integer);
begin
  FRedProcent := Value;
end;

end.
